# -*- coding: utf-8 -*-
"""
/***************************************************************************
 XYBL_TOOL for QGIS
                                 A QGIS plugin
        begin                : 2019-01-16
        copyright            : (C) 2019 by n-survey
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4 import QtGui, uic
from PyQt4.QtCore import *
from PyQt4.QtGui import *
import os
import sys
from qgis.core import *
from qgis.gui import *
import qgis.utils
import os
import sys
import codecs
import datetime
import time
from time import sleep
if sys.platform == "win32":
    from .dialog import Ui_dialog
else:
    from .dialog_Mac import Ui_dialog
import urllib.request, urllib.error, urllib.parse
import json



class XYBL_TOOL:
    def __init__(self, iface):
        self.iface = iface
        self.plugin_dir = os.path.dirname(__file__)
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'XYBL_TOOL_{}.qm'.format(locale))
        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)
        self.dlg = XYBL_TOOLDialog()
        self.actions = []
        self.menu = self.tr(u'&XYBL_TOOL')
        self.toolbar = self.iface.addToolBar(u'XYBL_TOOL')
        self.toolbar.setObjectName(u'XYBL_TOOL')

    def tr(self, message):
        return QCoreApplication.translate('XYBL_TOOL', message)

    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)
        if status_tip is not None:
            action.setStatusTip(status_tip)
        if whats_this is not None:
            action.setWhatsThis(whats_this)
        if add_to_toolbar:
            self.toolbar.addAction(action)
        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)
        self.actions.append(action)
        return action

    def initGui(self):
        self.add_action(
            icon_path=':/icon/earth64.png',
            text=self.tr(u'XYBL_TOOL'),
            callback=self.run,
            parent=self.iface.mainWindow())
        self.add_action(
            icon_path='',
            #text=self.tr(u'ウィンドウ位置初期化'),
            text=self.tr(u'Initialize'),
            add_to_toolbar=False,
            callback=self.run2,
            parent=self.iface.mainWindow())
        # マップキャンバスクリック時の動作
        self.mapTool = CaptureCoordinates(self.iface)

    def unload(self):
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&XYBL_TOOL'),
                action)
            self.iface.removeToolBarIcon(action)
        del self.toolbar

    def run(self):
        self.iface.mapCanvas().setMapTool(self.mapTool)
        # ウィンドウタイトル表示（バージョン取得）
        for line in open(os.path.dirname(__file__) + u'/metadata.txt', 'r'):
            if line[0:8] == 'version=':
                version = line.replace('version=', '')
        self.dlg.setWindowTitle('XYBL_TOOL for QGIS2.18 Ver.' + version)
        self.dlg.show()

    # ウィンドウ位置初期化
    def run2(self):
        ws = QSettings("M'z Room", "XYBL_TOOL2")
        ws.setValue("setting/window_w", 700)
        ws.setValue("setting/window_h", 368)
        ws.setValue("setting/window_x", 8)
        ws.setValue("setting/window_y", 31)



# マップキャンバスクリック時の動作
class CaptureCoordinates(QgsMapTool):
    def __init__(self, iface):
        QgsMapTool.__init__(self, iface.mapCanvas())
        self.iface = iface

    def canvasReleaseEvent(self, e):
        mPos = self.toMapCoordinates(e.pos())
        self.iface.mapCanvas().setCenter(mPos)
        self.iface.mapCanvas().refresh()
        # 現在の縮尺を取得
        scale = int(round(map.scale(), 0))
        xc = str(mPos.y())
        yc = str(mPos.x())
        coordinate = xc + ',' + yc + ',' + str(scale)
        # クリップボードに送る
        clipboard = QApplication.clipboard()
        clipboard.clear()
        clipboard.setText(coordinate)



class XYBL_TOOLDialog(QDialog):
    def __init__(self):
        QDialog.__init__(self)
        self.ui = Ui_dialog()
        self.ui.setupUi(self)
        # マップキャンバスウィジェット
        global map
        map = qgis.utils.iface.mapCanvas()
        ws = QSettings("M'z Room", "XYBL_TOOL2")
        window_w = ws.value("setting/window_w", 270)
        window_h = ws.value("setting/window_h", 500)
        window_x = ws.value("setting/window_x", 8)
        window_y = ws.value("setting/window_y", 32)
        self.setGeometry(window_x, window_y, window_w, window_h)
        # 平面直角座標系（zone）
        self.ui.zone.addItem(u'1系')
        self.ui.zone.addItem(u'2系')
        self.ui.zone.addItem(u'3系')
        self.ui.zone.addItem(u'4系')
        self.ui.zone.addItem(u'5系')
        self.ui.zone.addItem(u'6系')
        self.ui.zone.addItem(u'7系')
        self.ui.zone.addItem(u'8系')
        self.ui.zone.addItem(u'9系')
        self.ui.zone.addItem(u'10系')
        self.ui.zone.addItem(u'11系')
        self.ui.zone.addItem(u'12系')
        self.ui.zone.addItem(u'13系')
        self.ui.zone.addItem(u'14系')
        self.ui.zone.addItem(u'15系')
        self.ui.zone.addItem(u'16系')
        self.ui.zone.addItem(u'17系')
        self.ui.zone.addItem(u'18系')
        self.ui.zone.addItem(u'19系')
        kei = ws.value("setting/kei", 12)
        self.ui.zone.setCurrentIndex(kei)
        # 測地系（refFrame）
        self.ui.refFrame.addItem(u'日本測地系')
        self.ui.refFrame.addItem(u'世界測地系')
        self.ui.refFrame.setCurrentIndex(1)
        #  変換方向（sokuti）
        self.ui.sokuti.addItem(u'日本測地系→世界測地系／順方向の補正')
        self.ui.sokuti.addItem(u'世界測地系→日本測地系／逆方向の補正')
        self.ui.sokuti.setCurrentIndex(0)
        #  変換方向 (Place)
        self.ui.Place.addItem(u'緯度・経度')
        self.ui.Place.addItem(u'平面直角座標')
        self.ui.Place.setCurrentIndex(1)
        # 補正パラメータファイル（chiiki）
        self.ui.chiiki.addItem(u'tokachi2003.par')
        self.ui.chiiki.addItem(u'tokachi2003b.par')
        self.ui.chiiki.addItem(u'fukuoka2005.par')
        self.ui.chiiki.addItem(u'noto2007_BL.par')
        self.ui.chiiki.addItem(u'chuetsuoki2007.par')
        self.ui.chiiki.addItem(u'iwatemiyagi2008.par')
        self.ui.chiiki.addItem(u'miyakojima2008.par')
        self.ui.chiiki.addItem(u'touhokutaiheiyouoki2011.par')
        self.ui.chiiki.addItem(u'kumamoto2016_BL.par')
        self.ui.chiiki.setCurrentIndex(0)
        # プロジェクトのCRS
        self.ui.Project_CRS.addItem(u'世界測地系  1系：2443')
        self.ui.Project_CRS.addItem(u'世界測地系  2系：2444')
        self.ui.Project_CRS.addItem(u'世界測地系  3系：2445')
        self.ui.Project_CRS.addItem(u'世界測地系  4系：2446')
        self.ui.Project_CRS.addItem(u'世界測地系  5系：2447')
        self.ui.Project_CRS.addItem(u'世界測地系  6系：2448')
        self.ui.Project_CRS.addItem(u'世界測地系  7系：2449')
        self.ui.Project_CRS.addItem(u'世界測地系  8系：2450')
        self.ui.Project_CRS.addItem(u'世界測地系  9系：2451')
        self.ui.Project_CRS.addItem(u'世界測地系 10系：2452')
        self.ui.Project_CRS.addItem(u'世界測地系 11系：2453')
        self.ui.Project_CRS.addItem(u'世界測地系 12系：2454')
        self.ui.Project_CRS.addItem(u'世界測地系 13系：2455')
        self.ui.Project_CRS.addItem(u'世界測地系 14系：2456')
        self.ui.Project_CRS.addItem(u'世界測地系 15系：2457')
        self.ui.Project_CRS.addItem(u'世界測地系 16系：2458')
        self.ui.Project_CRS.addItem(u'世界測地系 17系：2459')
        self.ui.Project_CRS.addItem(u'世界測地系 18系：2460')
        self.ui.Project_CRS.addItem(u'世界測地系 19系：2461')
        self.ui.Project_CRS.addItem(u'日本測地系  1系：30161')
        self.ui.Project_CRS.addItem(u'日本測地系  2系：30162')
        self.ui.Project_CRS.addItem(u'日本測地系  3系：30163')
        self.ui.Project_CRS.addItem(u'日本測地系  4系：30164')
        self.ui.Project_CRS.addItem(u'日本測地系  5系：30165')
        self.ui.Project_CRS.addItem(u'日本測地系  6系：30166')
        self.ui.Project_CRS.addItem(u'日本測地系  7系：30167')
        self.ui.Project_CRS.addItem(u'日本測地系  8系：30168')
        self.ui.Project_CRS.addItem(u'日本測地系  9系：30169')
        self.ui.Project_CRS.addItem(u'日本測地系 10系：30170')
        self.ui.Project_CRS.addItem(u'日本測地系 11系：30171')
        self.ui.Project_CRS.addItem(u'日本測地系 12系：30172')
        self.ui.Project_CRS.addItem(u'日本測地系 13系：30173')
        self.ui.Project_CRS.addItem(u'日本測地系 14系：30174')
        self.ui.Project_CRS.addItem(u'日本測地系 15系：30175')
        self.ui.Project_CRS.addItem(u'日本測地系 16系：30176')
        self.ui.Project_CRS.addItem(u'日本測地系 17系：30177')
        self.ui.Project_CRS.addItem(u'日本測地系 18系：30178')
        self.ui.Project_CRS.addItem(u'日本測地系 19系：30179')
        self.ui.Project_CRS.addItem(u'世界測地系 JGD2000：4612')
        self.ui.Project_CRS.addItem(u'日本測地系 Tokyo：4301')
        self.setEPSG()
        # タイルリストを追加
        tfname = os.path.dirname(__file__) + u'/tile.txt'
        if sys.platform == "win32":
            if os.path.exists(tfname):
                for line in codecs.open(tfname,'r', 'utf-8'):
                    if line[:1] != '#':
                        dat = line.replace((u'\n' or u'\r\n'),'').split(',')
                        self.ui.haikei.addItem(dat[0])
                        self.ui.haikei2.addItem(line.replace((u'\n' or u'\r\n'),''))
        else:
            if os.path.exists(tfname):
                for line in codecs.open(tfname,'r', 'utf-8'):
                    if line[:1] != '#':
                        dat = line.split(',')
                        self.ui.haikei.addItem(dat[0])
                        self.ui.haikei2.addItem(line.replace((u'\n' or u'\r\n'),''))
        self.ui.x1.setFocus()
        # 現在の縮尺を取得
        scale = int(round(map.scale(), 0))
        self.ui.scale.setText(str(scale))
        # シグナル
        self.ui.kei_1.clicked.connect(self.keiSet1)
        self.ui.kei_2.clicked.connect(self.keiSet2)
        self.ui.kei_3.clicked.connect(self.keiSet3)
        self.ui.kei_4.clicked.connect(self.keiSet4)
        self.ui.kei_5.clicked.connect(self.keiSet5)
        self.ui.kei_6.clicked.connect(self.keiSet6)
        self.ui.kei_7.clicked.connect(self.keiSet7)
        self.ui.kei_8.clicked.connect(self.keiSet8)
        self.ui.kei_9.clicked.connect(self.keiSet9)
        self.ui.kei_10.clicked.connect(self.keiSet10)
        self.ui.kei_11.clicked.connect(self.keiSet11)
        self.ui.kei_12.clicked.connect(self.keiSet12)
        self.ui.kei_13.clicked.connect(self.keiSet13)
        self.ui.kei_14.clicked.connect(self.keiSet14)
        self.ui.kei_15.clicked.connect(self.keiSet15)
        self.ui.kei_16.clicked.connect(self.keiSet16)
        self.ui.kei_17.clicked.connect(self.keiSet17)
        self.ui.kei_18.clicked.connect(self.keiSet18)
        self.ui.kei_19.clicked.connect(self.keiSet19)
        self.ui.pushButton_CRS.clicked.connect(self.change_CRS)
        self.ui.pushButton_kei.clicked.connect(self.pButton_kei)
        self.ui.pushButton_Calc.clicked.connect(self.Calc)
        self.ui.CSV.clicked.connect(self.csv_save)
        self.ui.SIMA.clicked.connect(self.sima_save)
        self.ui.clear_log.clicked.connect(self.clearlog)
        self.ui.addLayer.clicked.connect(self.addTileLayer)
        self.ui.zone.currentIndexChanged.connect(self.SetProjectCRS)
        self.ui.refFrame.currentIndexChanged.connect(self.SetProjectCRS)
        self.ui.Place.currentIndexChanged.connect(self.SetProjectCRS)
        self.ui.sokuti.currentIndexChanged.connect(self.SetProjectCRS)
        self.ui.chiiki.currentIndexChanged.connect(self.SetProjectCRS)
        self.ui.radioButton_xybl.clicked.connect(self.SetProjectCRS)
        self.ui.radioButton_blxy.clicked.connect(self.SetProjectCRS)
        self.ui.radioButton_sokuchikei.clicked.connect(self.SetProjectCRS)
        self.ui.radioButton_jishin.clicked.connect(self.SetProjectCRS)
        self.ui.radioButton_renzoku.clicked.connect(self.SetProjectCRS)
        self.ui.check_plot.clicked.connect(self.SetProjectCRS)
        self.ui.check_renban.clicked.connect(self.renban)
        self.ui.haikei.currentIndexChanged.connect(self.SetTile)
        self.ui.check_scale.clicked.connect(self.SetCenter)
        self.ui.Project_CRS.currentIndexChanged.connect(self.SetCenter)

    # タイル地図リスト
    def SetTile(self):
        self.ui.haikei2.setCurrentIndex(self.ui.haikei.currentIndex())

    # フォーカスセット
    def renban(self):
        self.ui.pn.clear()
        if self.ui.check_renban.isChecked():
            self.ui.pn.setEnabled(False)
            self.ui.x1.setFocus()
        else:
            self.ui.pn.setEnabled(True)
            self.ui.pn.setFocus()

    # 平面直角座標系選択1
    def SetProjectCRS(self):
        if self.ui.flag.text() == '1':
            return
        self.ui.refFrame.setEnabled(False)
        self.ui.sokuti.setEnabled(False)
        self.ui.Place.setEnabled(False)
        self.ui.chiiki.setEnabled(False)
        # 平面直角座標→緯度・経度
        if self.ui.radioButton_xybl.isChecked():
            self.ui.refFrame.setEnabled(True)
        # 緯度・経度→平面直角座標
        elif self.ui.radioButton_blxy.isChecked():
            self.ui.refFrame.setEnabled(True)
        # 測地系変換・地震補正
        elif self.ui.radioButton_sokuchikei.isChecked():
            self.ui.sokuti.setEnabled(True)
            self.ui.Place.setEnabled(True)
        elif self.ui.radioButton_jishin.isChecked() or self.ui.radioButton_renzoku.isChecked():
            self.ui.sokuti.setEnabled(True)
            self.ui.Place.setEnabled(True)
            self.ui.chiiki.setEnabled(True)
        if self.ui.check_plot.isChecked():
            # 平面直角座標→緯度・経度
            if self.ui.radioButton_xybl.isChecked():
                self.ui.label_2.setText(u'Ｘ座標')
                self.ui.label_3.setText(u'Ｙ座標')
                if self.ui.refFrame.currentText() == u'世界測地系':
                    self.ui.Project_CRS.setCurrentIndex(38)
                else:
                    self.ui.Project_CRS.setCurrentIndex(39)
            # 緯度・経度→平面直角座標
            elif self.ui.radioButton_blxy.isChecked():
                self.ui.label_2.setText(u'緯　度')
                self.ui.label_3.setText(u'経　度')
                if self.ui.refFrame.currentText() == u'世界測地系':
                    self.ui.Project_CRS.setCurrentIndex(self.ui.zone.currentIndex())
                else:
                    self.ui.Project_CRS.setCurrentIndex(self.ui.zone.currentIndex() + 19)
            # 測地系変換・地震補正
            elif self.ui.radioButton_sokuchikei.isChecked() or self.ui.radioButton_jishin.isChecked() or self.ui.radioButton_renzoku.isChecked():
                if self.ui.Place.currentText() == u'平面直角座標':
                    self.ui.label_2.setText(u'Ｘ座標')
                    self.ui.label_3.setText(u'Ｙ座標')
                    if self.ui.sokuti.currentText() == u'日本測地系→世界測地系／順方向の補正':
                        self.ui.Project_CRS.setCurrentIndex(self.ui.zone.currentIndex())
                    else:
                        self.ui.Project_CRS.setCurrentIndex(self.ui.zone.currentIndex() + 19)
                else:
                    self.ui.label_2.setText(u'緯　度')
                    self.ui.label_3.setText(u'経　度')
                    if self.ui.sokuti.currentText() == u'日本測地系→世界測地系／順方向の補正':
                         self.ui.Project_CRS.setCurrentIndex(38)
                    else:
                         self.ui.Project_CRS.setCurrentIndex(39)
            self.change_CRS()
        if self.ui.check_renban.isChecked():
            self.ui.x1.setFocus()
        else:
            self.ui.pn.setFocus()

    # 平面直角座標系選択2
    def keiSet1(self):
        self.ui.zone.setCurrentIndex(0)
        self.SetProjectCRS()
    def keiSet2(self):
        self.ui.zone.setCurrentIndex(1)
        self.SetProjectCRS()
    def keiSet3(self):
        self.ui.zone.setCurrentIndex(2)
        self.SetProjectCRS()
    def keiSet4(self):
        self.ui.zone.setCurrentIndex(3)
        self.SetProjectCRS()
    def keiSet5(self):
        self.ui.zone.setCurrentIndex(4)
        self.SetProjectCRS()
    def keiSet6(self):
        self.ui.zone.setCurrentIndex(5)
        self.SetProjectCRS()
    def keiSet7(self):
        self.ui.zone.setCurrentIndex(6)
        self.SetProjectCRS()
    def keiSet8(self):
        self.ui.zone.setCurrentIndex(7)
        self.SetProjectCRS()
    def keiSet9(self):
        self.ui.zone.setCurrentIndex(8)
        self.SetProjectCRS()
    def keiSet10(self):
        self.ui.zone.setCurrentIndex(9)
        self.SetProjectCRS()
    def keiSet11(self):
        self.ui.zone.setCurrentIndex(10)
        self.SetProjectCRS()
    def keiSet12(self):
        self.ui.zone.setCurrentIndex(11)
        self.SetProjectCRS()
    def keiSet13(self):
        self.ui.zone.setCurrentIndex(12)
        self.SetProjectCRS()
    def keiSet14(self):
        self.ui.zone.setCurrentIndex(13)
        self.SetProjectCRS()
    def keiSet15(self):
        self.ui.zone.setCurrentIndex(14)
        self.SetProjectCRS()
    def keiSet16(self):
        self.ui.zone.setCurrentIndex(15)
        self.SetProjectCRS()
    def keiSet17(self):
        self.ui.zone.setCurrentIndex(16)
        self.SetProjectCRS()
    def keiSet18(self):
        self.ui.zone.setCurrentIndex(17)
        self.SetProjectCRS()
    def keiSet19(self):
        self.ui.zone.setCurrentIndex(18)
        self.SetProjectCRS()

    # プロジェクトのEPSG変更
    def change_CRS(self):
        dat = self.ui.Project_CRS.currentText().split(u'：')
        epsg = int(dat[1])
        custom = str(QgsCoordinateReferenceSystem(epsg).authid())
        if custom != '':#指定のCRSがカスタムCRSでない場合
            new_crs = QgsCoordinateReferenceSystem(epsg, QgsCoordinateReferenceSystem.EpsgCrsId)
        else:
            new_crs = QgsCoordinateReferenceSystem(epsg, QgsCoordinateReferenceSystem.InternalCrsId)
        map.setDestinationCrs(new_crs)
        if epsg ==4326 or epsg == 4612 or epsg == 4301:
            map.setMapUnits(2)
        else:
            map.setMapUnits(0)
        map.refresh()

    # プロジェクトのEPSGをセット
    def setEPSG(self):
        # プロジェクトのCRSを取得
        epsgGet()
        if epsgcode != '':
            code = epsgcode
        else:
            code = usercode
        n = 0
        try:
            for count in range(self.ui.Project_CRS.count()):
                dat = self.ui.Project_CRS.itemText(count).split(u'：')
                if int(dat[1]) == int(code):
                    n = count
        except:
            pass
        self.ui.Project_CRS.setCurrentIndex(n)
        map.refresh()

    # 変換
    def Calc(self):
        if self.ui.x1.text() == '' or self.ui.y1.text() == '':
            return
        # パラメータ一覧
        # outputType       =  出力タイプ（xml／json）
        # zone             =  平面直角座標系（1～19）
        # refFrame         =  測地系（1=日本測地系／2=世界測地系）
        # sokuti           =  変換方向（Tky2jgd：1=日本測地系→世界測地系／2=世界測地系→日本測地系 ｜ PatchJGD：0=順方向の補正／1=逆方向の補正）
        # Place            =  変換方法（Tky2jgd：1=緯度・経度で変換する／2=平面直角座標で変換する   ｜ PatchJGD：0=緯度・経度で変換する／1=平面直角座標で変換する）
        # chiiki           =  補正パラメータファイル（パラメータファイル名[*.par]）
        # publicX          =  Ｘ座標（m）
        # publicY          =  Ｙ座標（m）
        # latitude         =  緯度（degree）
        # longitude        =  経度（degree）
        # ellipsoid        =  準拠楕円体（GRS80／Bessel）
        # coordinateSystem =  座標系
        # gridConv         =  真北方向角（degree）
        # scaleFactor      =  縮尺係数
        # altitude         =  標高（m）
        # geoidHeight      =  ジオイド高（m）
        global publicX
        global publicY
        global latitude
        global longitude
        global gridConv
        global scaleFactor
        global zone
        global ellipsoid
        global altitude
        global geoidHeight
        global name
        global srcX
        global srcY
        global srcLat
        global srcLon
        name = ''
        srcX = ''
        srcY = ''
        srcLat = ''
        srcLon = ''
        ellipsoid = ''
        latitude = ''
        longitude = ''
        zone = ''
        publicX = ''
        publicY = ''
        altitude = ''
        geoidHeight = ''
        gridConv = ''
        scaleFactor = ''
        self.ui.groupBox_1.setEnabled(False)
        self.ui.groupBox_2.setEnabled(False)
        self.ui.kei_1.setEnabled(False)
        self.ui.kei_2.setEnabled(False)
        self.ui.kei_3.setEnabled(False)
        self.ui.kei_4.setEnabled(False)
        self.ui.kei_5.setEnabled(False)
        self.ui.kei_6.setEnabled(False)
        self.ui.kei_7.setEnabled(False)
        self.ui.kei_8.setEnabled(False)
        self.ui.kei_9.setEnabled(False)
        self.ui.kei_10.setEnabled(False)
        self.ui.kei_11.setEnabled(False)
        self.ui.kei_12.setEnabled(False)
        self.ui.kei_13.setEnabled(False)
        self.ui.kei_14.setEnabled(False)
        self.ui.kei_15.setEnabled(False)
        self.ui.kei_16.setEnabled(False)
        self.ui.kei_17.setEnabled(False)
        self.ui.kei_18.setEnabled(False)
        self.ui.kei_19.setEnabled(False)
        self.ui.zone.setEnabled(False)
        outputType = 'json'
        zone =  str(self.ui.zone.currentIndex() + 1)
        refFrame = str(self.ui.refFrame.currentIndex() + 1)
        chiiki = self.ui.chiiki.currentText()
        t1 = self.ui.lineEdit_time.text().replace('-',',').replace(' ',',').replace(':',',').split(',')
        t2 = str(datetime.datetime.now()).replace('-',',').replace(' ',',').replace(':',',').split(',')
        #y1 = int(t1[0])
        #m1 = int(t1[1])
        d1 = int(t1[2])
        h1 = int(t1[3])
        m1 = int(t1[4])
        s1 = float(t1[5])
        #y2 = int(t2[0])
        #m2 = int(t2[1])
        d2 = int(t2[2])
        h2 = int(t2[3])
        m2 = int(t2[4])
        s2 = float(t2[5])
        d = (d2 - d1) * 24 * 60 * 60
        h = (h2 - h1) * 60 * 60
        m = (m2 - m1) * 60
        s = s2 - s1
        delta = d + h + m + s
        self.ui.lineEdit_time.setText(str(datetime.datetime.now()).replace('-',',').replace(' ',',').replace(':',','))
        #print(delta)
        # 平面直角座標→緯度・経度
        if self.ui.radioButton_xybl.isChecked():
            if delta < 1.1:
                sleep(1.1 - delta)
            srcX = str(self.ui.x1.text())
            srcY = str(self.ui.y1.text())
            URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/surveycalc/xy2bl.pl?outputType=json&refFrame=" + refFrame + u"&zone=" + zone + u"&publicX=" + srcX + u"&publicY=" + srcY
        # 緯度・経度→平面直角座標
        elif self.ui.radioButton_blxy.isChecked():
            if delta < 1.1:
                 sleep(1.1 - delta)
            srcLat = str(self.ui.x1.text())
            srcLon = str(self.ui.y1.text())
            URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/surveycalc/bl2xy.pl?outputType=json&refFrame=" + refFrame + u"&zone=" + zone  + u"&latitude=" + srcLat + u"&longitude=" + srcLon
        # 測地系変換（Tky2JGD）
        elif self.ui.radioButton_sokuchikei.isChecked():
            if delta < 3.4:
                 sleep(3.4 - delta)
            sokuti = str(self.ui.sokuti.currentIndex() + 1)
            if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                srcLat = str(self.ui.x1.text())
                srcLon = str(self.ui.y1.text())
                URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/tky2jgd/tky2jgd.pl?outputType=json&sokuti=" + sokuti + u"&Place=1&latitude=" + srcLat + u"&longitude=" + srcLon
            else:# 平面直角座標による変換
                srcX = str(self.ui.x1.text())
                srcY = str(self.ui.y1.text())
                URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/tky2jgd/tky2jgd.pl?outputType=json&sokuti=" + sokuti + u"&Place=2&zone=" + zone + u"&publicX=" + srcX + u"&publicY=" + srcY
        # 地震補正（PatchJGD）
        elif self.ui.radioButton_jishin.isChecked():
            if delta < 3.4:
                 sleep(3.4 - delta)
            sokuti = str(self.ui.sokuti.currentIndex())
            if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                srcLat = str(self.ui.x1.text())
                srcLon = str(self.ui.y1.text())
                URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/patchjgd/patchjgd.php?outputType=json&sokuti=" + sokuti + u"&Place=0&chiiki=" + chiiki + u"&latitude=" + srcLat + u"&longitude=" + srcLon
            else:# 平面直角座標による変換
                srcX = str(self.ui.x1.text())
                srcY = str(self.ui.y1.text())
                URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/patchjgd/patchjgd.php?outputType=json&sokuti=" + sokuti + u"&Place=1&chiiki=" + chiiki + u"&zone=" + zone + u"&publicX=" + srcX + u"&publicY=" + srcY
        # 測地系変換（Tky2JGD）＋ 地震補正（PatchJGD）
        elif self.ui.radioButton_renzoku.isChecked():
            if delta < 3.4:
                 sleep(3.4 - delta)
            sokuti = str(self.ui.sokuti.currentIndex() + 1)
            # 測地系変換（Tky2JGD）
            if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                srcLat = str(self.ui.x1.text())
                srcLon = str(self.ui.y1.text())
                URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/tky2jgd/tky2jgd.pl?outputType=json&sokuti=" + sokuti + u"&Place=1&latitude=" + srcLat + u"&longitude=" + srcLon
            else:# 平面直角座標による変換
                srcX = str(self.ui.x1.text())
                srcY = str(self.ui.y1.text())
                URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/tky2jgd/tky2jgd.pl?outputType=json&sokuti=" + sokuti + u"&Place=2&zone=" + zone + u"&publicX=" + srcX + u"&publicY=" + srcY
            try:
                #print(URL)
                data_all = urllib.request.urlopen(URL)
                data = json.loads(data_all.read())
                OutputData  = str(data['OutputData'])
                OutputData = OutputData.replace('{', '')
                OutputData = OutputData.replace('}', '')
                OutputData = OutputData.replace("'", '')
                OutputData = OutputData.replace(' ', '')
                OutputData = OutputData.replace('publicX:', '')
                OutputData = OutputData.replace('publicY:', '')
                OutputData = OutputData.replace('latitude:', '')
                OutputData = OutputData.replace('longitude:', '')
                OutputData = OutputData.replace('gridConv:', '')
                OutputData = OutputData.replace('scaleFactor:', '')
                OutputData = OutputData.replace('zone:', '')
                OutputData = OutputData.replace('ellipsoid:', '')
                OutputData = OutputData.replace('refFrame:', '')
                OutputData = OutputData.replace(u'uu', '')
                data_all.close()
                dat = OutputData.split(u',')
                if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                    srcLat = dat[0]
                    srcLon = dat[1]
                else:# 平面直角座標による変換
                    srcX = dat[4]
                    srcY = dat[3]
                # 地震補正（PatchJGD）
                if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                    URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/patchjgd/patchjgd.php?outputType=json&sokuti=" + sokuti + u"&Place=0&chiiki=" + chiiki + u"&latitude=" + srcLat + u"&longitude=" + srcLon
                else:# 平面直角座標による変換
                    URL = "https://vldb.gsi.go.jp/sokuchi/surveycalc/tky2jgd/tky2jgd.pl?outputType=json&sokuti=" + sokuti + u"&Place=2&zone=" + zone + u"&publicX=" + srcX + u"&publicY=" + srcY
                #print(URL)
                data_all = urllib.request.urlopen(URL)
                data = json.loads(data_all.read())
                OutputData  = str(data['OutputData'])
                OutputData = OutputData.replace('{', '')
                OutputData = OutputData.replace('}', '')
                OutputData = OutputData.replace("'", '')
                OutputData = OutputData.replace(' ', '')
                OutputData = OutputData.replace('publicX:', '')
                OutputData = OutputData.replace('publicY:', '')
                OutputData = OutputData.replace('latitude:', '')
                OutputData = OutputData.replace('longitude:', '')
                OutputData = OutputData.replace('gridConv:', '')
                OutputData = OutputData.replace('scaleFactor:', '')
                OutputData = OutputData.replace('zone:', '')
                OutputData = OutputData.replace('ellipsoid:', '')
                OutputData = OutputData.replace('refFrame:', '')
                OutputData = OutputData.replace(u'uu', '')
                data_all.close()
                dat = OutputData.split(u',')
                # 結果表示
                if self.ui.check_renban.isChecked():
                    no = str(int(self.ui.lineEdit_2.text()) + 1)
                    self.ui.lineEdit_2.setText(no)
                    name = self.ui.lineEdit_1.text() + self.ui.lineEdit_2.text()
                else:
                    name = self.ui.pn.text()
                if name == '':
                    name = u'P'
                if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                    if self.ui.flag2.text() == '':
                        self.ui.result_text.clear()
                        self.ui.flag2.setText('1')
                        self.ui.result_text.append('name,srcLat,srcLon,refFrame,latitude,longitude')
                    ellipsoid = dat[2]
                    latitude = dat[0]
                    longitude = dat[1]
                    OutputData = name + ',' + srcLat + ',' + srcLon + ',' + ellipsoid + ',' + latitude + ',' + longitude
                else:# 平面直角座標による変換
                    if self.ui.flag2.text() == '':
                        self.ui.result_text.clear()
                        self.ui.flag2.setText('1')
                        self.ui.result_text.append('name,srcX,srcY,refFrame,latitude,longitude,zone,publicX,publicY,gridConv,scaleFactor')
                    ellipsoid = dat[0]
                    latitude = dat[5]
                    longitude = dat[2]
                    zone = dat[1]
                    publicX = dat[4]
                    publicY = dat[3]
                    gridConv = dat[6]
                    scaleFactor = dat[7]
                    OutputData = name + ',' + srcX + ',' + srcY + ',' + ellipsoid + ',' + latitude + ',' + longitude + ',' + zone + ',' + publicX + ',' + publicY + ',' + gridConv + ',' + scaleFactor
                self.ui.result_text.append(OutputData)
                self.ui.pn.clear()
                self.ui.x1.clear()
                self.ui.y1.clear()
                if self.ui.check_plot.isChecked():
                    self.plot()
                if self.ui.check_renban.isChecked():
                    self.ui.x1.setFocus()
                else:
                    self.ui.pn.setFocus()
            except:
                pass
            return
        #print(URL)
        try:
            data_all = urllib.request.urlopen(URL)
            data = json.loads(data_all.read())
            OutputData  = str(data['OutputData'])
            OutputData = OutputData.replace('{', '')
            OutputData = OutputData.replace('}', '')
            OutputData = OutputData.replace("'", '')
            OutputData = OutputData.replace(' ', '')
            OutputData = OutputData.replace('publicX:', '')
            OutputData = OutputData.replace('publicY:', '')
            OutputData = OutputData.replace('latitude:', '')
            OutputData = OutputData.replace('longitude:', '')
            OutputData = OutputData.replace('gridConv:', '')
            OutputData = OutputData.replace('scaleFactor:', '')
            OutputData = OutputData.replace('zone:', '')
            OutputData = OutputData.replace('ellipsoid:', '')
            OutputData = OutputData.replace('refFrame:', '')
            #OutputData = OutputData.replace('altitude:', '')
            #OutputData = OutputData.replace('geoidHeight:', '')
            OutputData = OutputData.replace(u'uu', '')
            data_all.close()
            dat = OutputData.split(u',')
            # 結果表示
            if self.ui.check_renban.isChecked():
                no = str(int(self.ui.lineEdit_2.text()) + 1)
                self.ui.lineEdit_2.setText(no)
                name = self.ui.lineEdit_1.text() + self.ui.lineEdit_2.text()
            else:
                name = self.ui.pn.text()
            if name == '':
                name = u'P'
            # 平面直角座標→緯度・経度
            if self.ui.radioButton_xybl.isChecked():
                if self.ui.flag2.text() == '':
                    self.ui.result_text.clear()
                    self.ui.flag2.setText('1')
                    self.ui.result_text.append('name,srcX,srcY,latitude,longitude,gridConv,scaleFactor')
                latitude = dat[0]
                longitude = dat[3]
                gridConv = dat[1]
                scaleFactor = dat[2]
                OutputData = name + ',' + srcX + ',' + srcY + ',' + latitude + ',' + longitude + ',' + gridConv + ',' + scaleFactor
            # 緯度・経度→平面直角座標
            elif self.ui.radioButton_blxy.isChecked():
                if self.ui.flag2.text() == '':
                    self.ui.result_text.clear()
                    self.ui.flag2.setText('1')
                    self.ui.result_text.append('name,srcLat,srcLon,publicX,publicY,gridConv,scaleFactor')
                publicX = dat[2]
                publicY = dat[3]
                gridConv = dat[0]
                scaleFactor = dat[1]
                OutputData = name + ',' + srcLat + ',' + srcLon + ',' + publicX + ',' + publicY + ',' + gridConv + ',' + scaleFactor
            # 測地系変換（Tky2JGD）
            elif self.ui.radioButton_sokuchikei.isChecked():
                if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                    if self.ui.flag2.text() == '':
                        self.ui.result_text.clear()
                        self.ui.flag2.setText('1')
                        self.ui.result_text.append('name,srcLat,srcLon,ellipsoid,latitude,longitude')
                    ellipsoid = dat[2]
                    latitude = dat[0]
                    longitude = dat[1]
                    OutputData = name + ',' + srcLat + ',' + srcLon+ ',' + ellipsoid + ',' + latitude + ',' + longitude
                else:# 平面直角座標による変換
                    if self.ui.flag2.text() == '':
                        self.ui.result_text.clear()
                        self.ui.flag2.setText('1')
                        self.ui.result_text.append('name,srcX,srcY,ellipsoid,latitude,longitude,zone,publicX,publicY,gridConv,scaleFactor')
                    ellipsoid = dat[0]
                    latitude = dat[5]
                    longitude = dat[2]
                    zone = dat[1]
                    publicX = dat[4]
                    publicY = dat[3]
                    gridConv = dat[6]
                    scaleFactor = dat[7]
                    OutputData = name + ',' + srcX + ',' + srcY + ',' + ellipsoid + ',' + latitude + ',' + longitude + ',' + zone + ',' + publicX + ',' + publicY + ',' + gridConv + ',' + scaleFactor
            # 地震補正（PatchJGD）
            elif self.ui.radioButton_jishin.isChecked():
                if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                    if self.ui.flag2.text() == '':
                        self.ui.result_text.clear()
                        self.ui.flag2.setText('1')
                        self.ui.result_text.append('name,srcLat,srcLon,refFrame,latitude,longitude')
                    ellipsoid = dat[2]
                    latitude = dat[0]
                    longitude = dat[1]
                    OutputData = name + ',' + srcLat + ',' + srcLon + ',' + ellipsoid + ',' + latitude + ',' + longitude
                else:# 平面直角座標による変換
                    if self.ui.flag2.text() == '':
                        self.ui.result_text.clear()
                        self.ui.flag2.setText('1')
                        self.ui.result_text.append('name,srcX,srcY,refFrame,latitude,longitude,zone,publicX,publicY,gridConv,scaleFactor')
                    ellipsoid = dat[3]
                    latitude = dat[5]
                    longitude = dat[1]
                    zone = dat[0]
                    publicX = dat[4]
                    publicY = dat[2]
                    gridConv = dat[6]
                    scaleFactor = dat[7]
                    OutputData = name + ',' + srcX + ',' + srcY + ',' + ellipsoid + ',' + latitude + ',' + longitude + ',' + zone + ',' + publicX + ',' + publicY + ',' + gridConv + ',' + scaleFactor
            self.ui.result_text.append(OutputData)
            self.ui.pn.clear()
            self.ui.x1.clear()
            self.ui.y1.clear()
            if self.ui.check_plot.isChecked():
                self.plot()
            if self.ui.check_renban.isChecked():
                self.ui.x1.setFocus()
            else:
                self.ui.pn.setFocus()
        except:
            pass

    # プロット
    def plot(self):
        # プロジェクトのCRSを取得
        epsgGet()
        layer = QgsVectorLayer('Point?crs=epsg:' + epsgcode + '&field=name&field=srcX&field=srcY&field=srcLat&field=srcLon&field=ellipsoid&field=latitude&field=longitude&field=zoon&field=publicX&field=publicYe&field=altitude&field=geoidHeight&field=gridConv&field=scaleFactor', name, 'memory')
        provider = layer.dataProvider()
        feature = QgsFeature()
        # 平面直角座標→緯度・経度
        if self.ui.radioButton_xybl.isChecked():
            geometry = QgsGeometry.fromPoint(QgsPoint(float(longitude), float(latitude)))
            rect = QgsRectangle(float(longitude), float(latitude),float(longitude), float(latitude))
        # 緯度・経度→平面直角座標
        elif self.ui.radioButton_blxy.isChecked():
            geometry = QgsGeometry.fromPoint(QgsPoint(float(publicY), float(publicX)))
            rect = QgsRectangle(float(publicY), float(publicX),float(publicY), float(publicX))
        # 測地系変換（Tky2JGD）
        elif self.ui.radioButton_sokuchikei.isChecked():
            if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                geometry = QgsGeometry.fromPoint(QgsPoint(float(longitude), float(latitude)))
                rect = QgsRectangle(float(longitude), float(latitude),float(longitude), float(latitude))
            else:# 平面直角座標による変換
                geometry = QgsGeometry.fromPoint(QgsPoint(float(publicY), float(publicX)))
                rect = QgsRectangle(float(publicY), float(publicX),float(publicY), float(publicX))
        # 地震補正（PatchJGD）
        elif self.ui.radioButton_jishin.isChecked() or self.ui.radioButton_renzoku.isChecked():
            if self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
                geometry = QgsGeometry.fromPoint(QgsPoint(float(longitude), float(latitude)))
                rect = QgsRectangle(float(longitude), float(latitude),float(longitude), float(latitude))
            else:# 平面直角座標による変換
                geometry = QgsGeometry.fromPoint(QgsPoint(float(publicY), float(publicX)))
                rect = QgsRectangle(float(publicY), float(publicX),float(publicY), float(publicX))
        feature.setGeometry(geometry)
        feature.setAttributes([name, srcX, srcY, srcLat, srcLon, ellipsoid, latitude, longitude, zone, publicX, publicY, altitude, geoidHeight, gridConv, scaleFactor])
        provider.addFeatures([feature])
        QgsMapLayerRegistry.instance().addMapLayer(layer, False)
        if QgsProject.instance().layerTreeRoot().findGroup(self.tr('Result')) == None:
            QgsProject.instance().layerTreeRoot().insertChildNode(0,QgsLayerTreeGroup(self.tr('Result')))
        group = QgsProject.instance().layerTreeRoot().findGroup(self.tr('Result'))
        group.insertLayer(0,layer)
        map.setExtent(rect)
        return

    # CSV保存
    def csv_save(self):
        if self.ui.flag2.text() == '':
            return
        ws = QSettings("M'z Room", "XYBL_TOOL2")
        reg_path = ws.value("setting/reg_path", os.path.dirname(__file__))
        dialog = QFileDialog()
        dialog.setFileMode(QFileDialog.Directory)
        try:
            dialog.setDirectory(reg_path)
        except:
            dialog.setDirectory(os.path.dirname(__file__))
        savename = dialog.getSaveFileName(self, u'CSV保存', '', '*.csv')
        if savename == "":
            return
        reg_path = os.path.dirname(str(savename))
        ws.setValue("setting/reg_path", reg_path)
        fname = open(savename, 'w')
        fname.write(self.ui.result_text.toPlainText() + '\n')
        fname.close()
        QMessageBox.information(self, u'CSV保存', u'「' + savename + u'」を作成しました。')

    # SIMA保存
    def sima_save(self):
        if self.ui.flag2.text() == '':
            return
        # 測地系変換（Tky2JGD）
        if self.ui.radioButton_sokuchikei.isChecked() and self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
            return
        # 地震補正（PatchJGD）
        if self.ui.radioButton_jishin.isChecked() and self.ui.Place.currentIndex() == 0:# 緯度・経度による変換
            return
        infile = os.path.dirname(__file__) + u'/tmp.csv'
        fname = codecs.open(infile, 'w', 'utf-8')
        fname.write(self.ui.result_text.toPlainText())
        fname.close()
        ws = QSettings("M'z Room", "XYBL_TOOL2")
        reg_path = ws.value("setting/reg_path", os.path.dirname(__file__))
        dialog = QFileDialog()
        dialog.setFileMode(QFileDialog.Directory)
        try:
            dialog.setDirectory(reg_path)
        except:
            dialog.setDirectory(os.path.dirname(__file__))
        savename = dialog.getSaveFileName(self, u'SIMA保存', '', '*.sim')
        if savename == "":
            return
        reg_path = os.path.dirname(str(savename))
        ws.setValue("setting/reg_path", reg_path)
        outfile = open(savename, 'w')
        n = 0
        outfile.write('G00,01,XYBL_TOOL for QGIS2.x,\n')
        outfile.write('Z00,座標ﾃﾞｰﾀ,\n')
        outfile.write('A00,\n')
        if os.path.exists(infile):
            for line in codecs.open(infile, 'r','utf-8'):
                if n > 0:
                    dat = line.split(',')
                    # 平面直角座標→緯度・経度
                    if self.ui.radioButton_xybl.isChecked():
                        p1 = dat[0]
                        x1 = dat[1]
                        y1 = dat[2]
                    # 緯度・経度→平面直角座標
                    elif self.ui.radioButton_blxy.isChecked():
                        p1 = dat[0]
                        x1 = dat[3]
                        y1 = dat[4]
                    # 測地系変換（Tky2JGD）・地震補正（PatchJGD）
                    if (self.ui.radioButton_sokuchikei.isChecked() or self.ui.radioButton_jishin.isChecked()) and self.ui.Place.currentIndex() == 1:# 平面直角座標による変換
                        p1 = dat[0]
                        x1 = dat[7]
                        y1 = dat[8]
                    outfile.write('A01,' + str(n) + ',' + p1 + ',' + x1 + ',' + y1 + ',,\n')
                n = n + 1
            outfile.write('A99,\n')
            outfile.close()
            QMessageBox.information(self, u'SIMA保存', u'「' + savename + u'」を作成しました。')

    # 消去
    def clearlog(self):
        kakunin = QMessageBox.question(self,u'消去', u'保存していない計算結果は失われます。\n実行してもよろしいですか？',QMessageBox.Yes | QMessageBox.No,QMessageBox.No)
        if kakunin == QMessageBox.Yes:
            self.ui.pn.clear()
            self.ui.x1.clear()
            self.ui.y1.clear()
            self.ui.result_text.clear()
            self.ui.lineEdit_2.setText('0')
            self.ui.groupBox_1.setEnabled(True)
            self.ui.groupBox_2.setEnabled(True)
            self.ui.kei_1.setEnabled(True)
            self.ui.kei_2.setEnabled(True)
            self.ui.kei_3.setEnabled(True)
            self.ui.kei_4.setEnabled(True)
            self.ui.kei_5.setEnabled(True)
            self.ui.kei_6.setEnabled(True)
            self.ui.kei_7.setEnabled(True)
            self.ui.kei_8.setEnabled(True)
            self.ui.kei_9.setEnabled(True)
            self.ui.kei_10.setEnabled(True)
            self.ui.kei_11.setEnabled(True)
            self.ui.kei_12.setEnabled(True)
            self.ui.kei_13.setEnabled(True)
            self.ui.kei_14.setEnabled(True)
            self.ui.kei_15.setEnabled(True)
            self.ui.kei_16.setEnabled(True)
            self.ui.kei_17.setEnabled(True)
            self.ui.kei_18.setEnabled(True)
            self.ui.kei_19.setEnabled(True)
            self.ui.zone.setEnabled(True)
            self.ui.flag2.clear()
            self.ui.result_text.append(u'地理院タイル について')
            self.ui.result_text.append(u'https://maps.gsi.go.jp/development/ichiran.html')
            self.ui.result_text.append(u'今昔マップ on the web について')
            self.ui.result_text.append(u'http://ktgis.net/kjmapw/tilemapservice.html')
            self.ui.result_text.append(u'CS立体図 について')
            self.ui.result_text.append(u'http://kouapp.main.jp/csmap/japan/setumei.html')
            self.ui.result_text.append(u'OpenStreetMap について')
            self.ui.result_text.append(u'https://www.openstreetmap.org/copyright')

    # タイル地図をレイヤに追加
    def addTileLayer(self):
        # 現在のスケールを取得
        scale = int(self.ui.scale.text())
        # 変換先のEPSGを取得
        crs = self.ui.Project_CRS.currentText().split(u'：')
        epsg = int(crs[1])
        # タイル地図情報取得
        dat = self.ui.haikei2.currentText().split(u',')
        title = dat[0]
        #fmt = dat[1]
        #src = dat[2]
        zl = dat[3]
        z = zl.split('to')
        zmin = z[0].replace('z', '')
        zmax = z[1]
        url = dat[4].replace('"', '')
        minx = dat[5]
        miny = dat[6]
        maxx = dat[7]
        maxy = dat[8]
        x = self.ui.x1.text()
        y = self.ui.y1.text()
        # オンザフライ投影変換を有効にする
        map.mapRenderer().setProjectionsEnabled(True)
        # タイル地図を読み込む
        uri = 'url=' + url + '&zmax=' + zmax + '&zmin=' + zmin + '&type=xyz'
        layer = QgsRasterLayer(uri, title, 'wms')
        if not layer.isValid():
            QMessageBox.information(self, u'エラー', u'読み込みに失敗しました。')
            return
        else:
            QgsMapLayerRegistry.instance().addMapLayer(layer)
            QApplication.instance().processEvents()
            rect = QgsRectangle(float(miny), float(minx), float(maxy), float(maxx))
            map.setExtent(rect)
            # 指定のEPSGに変更
            self.change_CRS()
            # 領域の座標をEPSG:3857から指定のEPSGに変更
            crsSrc = QgsCoordinateReferenceSystem(3857)
            custom = str(QgsCoordinateReferenceSystem(epsg).authid())
            if custom == '':# 変換先がカスタムCRSの場合
                Dest_crs = QgsCoordinateReferenceSystem()
                Dest_crs.createFromId(epsg, QgsCoordinateReferenceSystem.InternalCrsId)
                crsDest = QgsCoordinateReferenceSystem(Dest_crs)
            else:
                crsDest = QgsCoordinateReferenceSystem(epsg)
            trans = QgsCoordinateTransform(crsSrc, crsDest)
            miny,minx = trans.transform(QgsPoint(float(miny), float(minx)))
            maxy,maxx = trans.transform(QgsPoint(float(maxy), float(maxx)))
            rect = QgsRectangle(float(miny), float(minx), float(maxy), float(maxx))
            map.setExtent(rect)
        # 中心座標の指定がある場合
        if self.ui.x1.text() != '' and self.ui.y1.text() != '':
            rect = QgsRectangle(float(y), float(x), float(y), float(x))
            map.setExtent(rect)
            map.zoomScale(scale)
        # 現在の縮尺を取得する
        scale = int(round(map.scale(), 0))
        self.ui.scale.setText(str(scale))
        # 平面直角座標系の場合は、左上の座標系リストも変更する
        self.ui.flag.setText('1')
        if (epsg - 2443) >= 0 and (epsg - 2443) < 19:
             self.ui.zone.setCurrentIndex(epsg - 2443)
        elif  (epsg - 30161) >= 0 and (epsg - 30161) < 19:
             self.ui.zone.setCurrentIndex(epsg - 30161)
        self.ui.flag.clear()

    def SetCenter(self):#マップキャンバスの中心座標を取得
        if self.ui.check_scale.isChecked():
            # 現在の縮尺を取得
            scale = int(round(map.scale(), 0))
            self.ui.scale.setText(str(scale))
            # 変換先のEPSGを取得
            crs = self.ui.Project_CRS.currentText().split(u'：')
            epsg = int(crs[1])
            # マップキャンバスの中心座標を取得
            mPos = map.center()
            # プロジェクトのEPSGを取得
            epsgGet()
            if epsgcode != '':
                crsSrc = QgsCoordinateReferenceSystem(int(epsgcode))
            else:
                Src_crs = QgsCoordinateReferenceSystem()
                Src_crs.createFromId(int(usercode), QgsCoordinateReferenceSystem.InternalCrsId)
                crsSrc = QgsCoordinateReferenceSystem(Src_crs)
            # マップキャンバスの中心座標を指定のEPSGに変更
            custom = str(QgsCoordinateReferenceSystem(epsg).authid())
            if custom == '':# 変換先がカスタムCRSの場合
                Dest_crs = QgsCoordinateReferenceSystem()
                Dest_crs.createFromId(epsg, QgsCoordinateReferenceSystem.InternalCrsId)
                crsDest = QgsCoordinateReferenceSystem(Dest_crs)
            else:
                crsDest = QgsCoordinateReferenceSystem(epsg)
            trans = QgsCoordinateTransform(crsSrc, crsDest)
            yc,xc = trans.transform(QgsPoint(mPos.x(), mPos.y()))
            self.ui.x1.setText(str(xc))
            self.ui.y1.setText(str(yc))
        else:
            self.ui.x1.clear()
            self.ui.y1.clear()
            # 現在の縮尺を取得
            scale = int(round(map.scale(), 0))
            self.ui.scale.setText(str(scale))

    # ×ボタンがクリックされた時
    def closeEvent(self,event):
        ws = QSettings("M'z Room", "XYBL_TOOL2")
        ws.setValue("setting/window_w", self.width())             #ウィンドウの幅
        ws.setValue("setting/window_h", self.height())            #ウィンドウの高さ
        ws.setValue("setting/window_x", self.x() + 8)             #ウィンドウの横位置
        ws.setValue("setting/window_y", self.y() + 31)            #ウィンドウの縦位置
        ws.setValue("setting/kei", self.ui.zone.currentIndex())   #平面直角座標系

    # EPSGコード(I/O、GDAL/OGR、Georeference、座標変換)
    def pButton_kei(self):
        if sys.platform == "win32":
            dat = u'【1系】\n'
            dat = dat + u'長崎県 鹿児島県のうち北方北緯32度南方北緯27度西方東経128度18分東方東経130度を境界線とする区域内（奄美群島は東経130度13分までを含む)にあるすべての島、小島、環礁及び岩礁\n'
            dat = dat + u'【2系】\n'
            dat = dat + u'福岡県 佐賀県 熊本県 大分県 宮崎県 鹿児島県（第Ｉ系に規定する区域を除く) \n'
            dat = dat + u'【3系】\n'
            dat = dat + u'山口県 島根県 広島県\n'
            dat = dat + u'【4系】\n'
            dat = dat + u'香川県 愛媛県 徳島県 高知県\n'
            dat = dat + u'【5系】\n'
            dat = dat + u'兵庫県 鳥取県 岡山県\n'
            dat = dat + u'【6系】\n'
            dat = dat + u'京都府 大阪府 福井県 滋賀県 三重県 奈良県 和歌山県\n'
            dat = dat + u'【7系】\n'
            dat = dat + u'石川県 富山県 岐阜県 愛知県\n'
            dat = dat + u'【8系】\n'
            dat = dat + u'新潟県 長野県 山梨県 静岡県\n'
            dat = dat + u'【9系】\n'
            dat = dat + u'東京都（XIV系、XVIII系及びXIX系に規定する区域を除く) 福島県 栃木県 茨城県 埼玉県 千葉県 群馬県 神奈川県\n'
            dat = dat + u'【10系】\n'
            dat = dat + u'青森県 秋田県 山形県 岩手県 宮城県\n'
            dat = dat + u'【11系】\n'
            dat = dat + u'小樽市 函館市 伊達市 胆振支庁管内のうち有珠郡及び虻田郡 檜山支庁管内 後志支庁管内 渡島支庁管内\n'
            dat = dat + u'【12系】\n'
            dat = dat + u'札幌市 旭川市 稚内市 留萌市 美唄市 夕張市 岩見沢市 苫小牧市 室蘭市 士別市 名寄市 芦別市 赤平市 三笠市 滝川市 砂川市 江別市 千歳市 歌志内市 深川市 紋別市 富良野市 登別市 恵庭市 北広島市 石狩市 石狩支庁管内 網走支庁管内のうち紋別郡上川支庁管内 宗谷支庁管内 日高支庁管内 胆振支庁管内（有珠郡及び虻田郡を除く。) 空知支庁管内 留萌支庁管内\n'
            dat = dat + u'【13系】\n'
            dat = dat + u'北見市 帯広市 釧路市 網走市 根室市 根室支庁管内 釧路支庁管内 網走支庁管内（紋別郡を除く。）十勝支庁管内\n'
            dat = dat + u'【14系】\n'
            dat = dat + u'東京都のうち北緯28度から南であり、かつ東経140度30分から東であり東経143度から西である区域\n'
            dat = dat + u'【15系】\n'
            dat = dat + u'沖縄県のうち東経126度から東であり、かつ東経130度から西である区域\n'
            dat = dat + u'【16系】\n'
            dat = dat + u'沖縄県のうち東経126度から西である区域\n'
            dat = dat + u'【17系】\n'
            dat = dat + u'沖縄県のうち東経130度から東である区域\n'
            dat = dat + u'【18系】\n'
            dat = dat + u'東京都のうち北緯28度から南であり、かつ東経140度30分から西である区域\n'
            dat = dat + u'【19系】\n'
            dat = dat + u'東京都のうち北緯28度から南であり、かつ東経143度から東である区域'
        else:
            dat = u'【1系】長崎県 鹿児島県のうち北方北緯32度南方北緯27度西方東経128度18分東方東経130度を境界線とする区域内（奄美群島は東経130度13分までを含む)にあるすべての島、小島、環礁及び岩礁【2系】福岡県 佐賀県 熊本県 大分県 宮崎県 鹿児島県（第Ｉ系に規定する区域を除く) 【3系】山口県 島根県 広島県【4系】香川県 愛媛県 徳島県 高知県【5系】兵庫県 鳥取県 岡山県【6系】京都府 大阪府 福井県 滋賀県 三重県 奈良県 和歌山県【7系】石川県 富山県 岐阜県 愛知県【8系】新潟県 長野県 山梨県 静岡県【9系】東京都（XIV系、XVIII系及びXIX系に規定する区域を除く) 福島県 栃木県 茨城県 埼玉県 千葉県 群馬県 神奈川県【10系】青森県 秋田県 山形県 岩手県 宮城県【11系】小樽市 函館市 伊達市 胆振支庁管内のうち有珠郡及び虻田郡 檜山支庁管内 後志支庁管内 渡島支庁管内【12系】札幌市 旭川市 稚内市 留萌市 美唄市 夕張市 岩見沢市 苫小牧市 室蘭市 士別市 名寄市 芦別市 赤平市 三笠市 滝川市 砂川市 江別市 千歳市 歌志内市 深川市 紋別市 富良野市 登別市 恵庭市 北広島市 石狩市 石狩支庁管内 網走支庁管内のうち紋別郡上川支庁管内 宗谷支庁管内 日高支庁管内 胆振支庁管内（有珠郡及び虻田郡を除く。) 空知支庁管内 留萌支庁管内【13系】北見市 帯広市 釧路市 網走市 根室市 根室支庁管内 釧路支庁管内 網走支庁管内（紋別郡を除く。）十勝支庁管内【14系】東京都のうち北緯28度から南であり、かつ東経140度30分から東であり東経143度から西である区域【15系】沖縄県のうち東経126度から東であり、かつ東経130度から西である区域【16系】沖縄県のうち東経126度から西である区域【17系】沖縄県のうち東経130度から東である区域【18系】東京都のうち北緯28度から南であり、かつ東経140度30分から西である区域【19系】\東京都のうち北緯28度から南であり、かつ東経143度から東である区域'
        QMessageBox.information(self, u'平面直角座標系について', dat)



# プロジェクトのEPSG取得
def epsgGet():
    global epsgcode
    global usercode
    crs = qgis.utils.iface.mapCanvas().mapSettings().destinationCrs()
    epsgcode = str(crs.authid())
    if epsgcode[0:5] == 'EPSG:':
      epsgcode = epsgcode.replace('EPSG:', '')
      usercode = ''
    else:
      usercode = epsgcode.replace('USER:', '')
      epsgcode = ''
